local VERSION = tonumber(("$Revision: 33596 $"):match("%d+"))

local PitBull = PitBull
local PitBull_RaidTarget = PitBull:NewModule("RaidTarget", "AceEvent-2.0")
local self = PitBull_RaidTarget
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-04-24 04:27:05 -0400 (Tue, 24 Apr 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local L = PitBull:L("PitBull-RaidTarget")

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame

PitBull_RaidTarget:RegisterPitBullChildFrames('raidTargetIcon')

function PitBull_RaidTarget:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("RaidTarget")
	PitBull:RegisterDefaults("RaidTarget", "profile", {
		['*'] = {
			disable = false,
		}
	})
end

function PitBull_RaidTarget:OnEnable()
	self:RegisterEvent("RAID_TARGET_UPDATE")
	self:RegisterEvent("RAID_ROSTER_UPDATE", "RAID_TARGET_UPDATE")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "RAID_TARGET_UPDATE")
end

function PitBull_RaidTarget:Update(unit, frame)
	local index = GetRaidTargetIndex(unit)
	if self.db.profile[frame.group].disable then
		index = nil
	end
	if index then
		if not frame.raidTargetIcon then
			frame.raidTargetIcon = newFrame("Texture", frame.overlay, "ARTWORK")
			frame.raidTargetIcon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
			PitBull:UpdateLayout(frame)
		end
		
		SetRaidTargetIconTexture(frame.raidTargetIcon, index)
	else
		if frame.raidTargetIcon then
			frame.raidTargetIcon = delFrame(frame.raidTargetIcon)
			
			PitBull:UpdateLayout(frame)
		end
	end
end

function PitBull_RaidTarget:RAID_TARGET_UPDATE()
	for unit, frame in PitBull:IterateNonWackyUnitFrames() do
		self:Update(unit, frame)
	end
end

function PitBull_RaidTarget:OnUpdateFrame(unit, frame)
	self:Update(unit, frame)
end

function PitBull_RaidTarget:OnClearUnitFrame(unit, frame)
	if frame.raidTargetIcon then
		frame.raidTargetIcon = delFrame(frame.raidTargetIcon)
	end
end

function PitBull_RaidTarget:OnUnknownLayout(unit, frame)
	frame.raidTargetIcon:SetPoint("CENTER", frame, "TOP", 0, -3)
	frame.raidTargetIcon:SetWidth(18)
	frame.raidTargetIcon:SetHeight(18)
end

local function getDisable(group)
	return PitBull_RaidTarget.db.profile[group].disable
end
local function setDisable(group, value)
	PitBull_RaidTarget.db.profile[group].disable = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_RaidTarget:Update(unit, frame)
	end
end
PitBull_RaidTarget:RegisterPitBullOptionsMethod(function(group)
	return {
		type = "toggle",
		name = L["Hide Raid Target Icon"],
		desc = L["Disable Raid Target Icon display on the unit frames for this unit type."],
		get = getDisable,
		set = setDisable,
		passValue = group,
	}
end)
