local VERSION = tonumber(("$Revision: 33626 $"):match("%d+"))

local PitBull = PitBull
local PitBull_Reputation = PitBull:NewModule("Reputation", "AceEvent-2.0", "AceConsole-2.0", "AceHook-2.1")
local self = PitBull_Reputation
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-04-24 06:15:11 -0400 (Tue, 24 Apr 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

-- localization placeholder
local L = PitBull:L("PitBull-Reputation")

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame

local repColor = {
	[1] = {r = 226/255, g = 45/255, b = 75/255}, -- Hated
	[2] = {r = 226/255, g = 45/255, b = 75/255}, -- Hostile
	[3] = {r = 0.75, g = 0.27, b = 0}, -- unfriendly
	[4] = {r = 1, g = 1, b = 34/255}, -- neutral
	[5] = {r = 0.2, g = 0.8, b = 0.15}, -- friendly
	[6] = {r = 0.2, g = 0.8, b = 0.15}, -- honored
	[7] = {r = 0.2, g = 0.8, b = 0.15}, -- revered
	[8] = {r = 0.2, g = 0.8, b = 0.15}, -- exalted
}

PitBull_Reputation:RegisterPitBullChildFrames('repBar', 'repBarText')

function PitBull_Reputation:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("Reputation")
	PitBull:RegisterDefaults("Reputation", "profile", {
		ignore = true,
		autoHide = false,
		textColor = {1, 1, 1},
		backgroundColor = { 1, 0, 0 },
		deficit = false,
		reverse = false,
		customColor = false,
		customColors = {1,1,1},
		customBGColor = false,
		customBGColors = {11/255, 11/255, 11/255},
	})
end

local repname, repreaction, repmin, repmax, repvalue = nil, 0, 0, 0, 0

function PitBull_Reputation:UpdateReputation()
	local frame = PitBull:GetUnitFrame("player")
	if not frame then
		return
	end
	if self.db.profile.ignore then
		if frame.repBar then
			PitBull_Reputation:OnClearUnitFrame("player", frame)
			PitBull:UpdateLayout(frame)
		end
		return
	else
		if not frame.repBar then
			PitBull_Reputation:OnPopulateUnitFrame("player", frame)
			PitBull:UpdateLayout(frame)
		end
	end
	repname, repreaction, repmin, repmax, repvalue = GetWatchedFactionInfo()
	if repname then
		if not frame.repBar then
			self:OnPopulateUnitFrame("player", frame)
			PitBull:UpdateLayout(frame)
		end
		repmax = repmax - repmin
		repvalue = repvalue - repmin
		repmin = 0
		if self.db.profile.deficit ~= self.db.profile.reverse then
			frame.repBar:SetValue((repmax - repvalue) / (repmax - repmin))
		else
			frame.repBar:SetValue(repvalue / (repmax - repmin))
		end
		local color = repColor[repreaction]
		local r, g, b
		if self.db.profile.customColor then
			r, g, b = unpack(self.db.profile.customColors)
		else
			r, g, b = color.r, color.g, color.b
		end
		local br, bg, bb
		if self.db.profile.customBGColor then
			br, bg, bb = unpack(self.db.profile.customBGColors)
		else
			br, bg, bb = (r+0.2)/3, (g+0.2)/3, (b+0.2)/3
		end
		if self.db.profile.reverse then
			frame.repBar:SetStatusBarColor(br, bg, bb, 1)
			frame.repBar.bg:SetVertexColor(r, g, b, 1)
		else
			frame.repBar:SetStatusBarColor(r, g, b, 1)
			frame.repBar.bg:SetVertexColor(br, bg, bb, 1)
		end
		frame.repBarText:SetText(("%d/%d (%.1f%%)"):format(repvalue, repmax, (repvalue/repmax)*100))
		frame.repBarText.reptext = frame.repBarText:GetText()
		frame.repBarText.repname = repname
		frame.repBarText:SetAlpha(1.0)
		frame.repBarText:SetTextColor(unpack(self.db.profile.textColor))
	else
		frame.repBarText.repname = nil
		if self.db.profile.autoHide then
			if frame.repBar then
				PitBull_Reputation:OnClearUnitFrame("player", frame)
				PitBull:UpdateLayout(frame)
			end
		else
			if frame.repBar then
				self:OnPopulateUnitFrame("player", frame)
				PitBull:UpdateLayout(frame)
			end
			frame.repBar.bg:SetVertexColor(unpack(self.db.profile.backgroundColor))
			frame.repBar:SetValue(0)
			frame.repBarText:SetText("No reputation selected")
			frame.repBarText:SetAlpha(1.0)
			frame.repBarText:SetTextColor(unpack(self.db.profile.textColor))
		end
	end
end

function PitBull_Reputation:OnEnable(first)
	if not self:IsHooked("ReputationWatchBar_Update") then
		self:SecureHook("ReputationWatchBar_Update", "UpdateReputation")
	end
end

function PitBull_Reputation:OnUpdateFrame(unit, frame)
	if unit == "player" then
		self:UpdateReputation()
	end
end

function PitBull_Reputation:OnFrameOnEnter(unit, frame)
	if frame.repBarText and frame.repBarText.repname then
		frame.repBarText:SetText(tostring(frame.repBarText.repname))
	end
end


function PitBull_Reputation:OnFrameOnLeave(unit, frame)
	if frame.repBarText and frame.repBarText.repname then
		frame.repBarText:SetText(tostring(frame.repBarText.reptext))
	end
end

function PitBull_Reputation:OnPopulateUnitFrame(unit, frame)
	if frame.repBar or self.db.profile.ignore then
		return 
	end
	if unit == "player" then
		local repBar = newFrame("StatusBar", frame)
		frame.repBar = repBar
		repBar:SetParent(frame)
		repBar:SetMinMaxValues(0, 1)
		repBar:SetValue(0.5)
		repBar:SetStatusBarTexture(PitBull:GetStatusBarTexture())
		repBar:SetStatusBarColor(0, 1, 0, 1)

		local repBarBG = newFrame("Texture", frame.repBar, "BACKGROUND")
		frame.repBar.bg = repBarBG
		repBarBG:SetTexture(PitBull:GetStatusBarTexture())
		repBarBG:SetVertexColor(unpack(self.db.profile.backgroundColor))
		repBarBG:ClearAllPoints()
		repBarBG:SetAllPoints(repBar)

		local font, fontsize = PitBull:GetFont()
		
		local repBarText = newFrame("FontString", repBar, "OVERLAY")
		frame.repBarText = repBarText
		repBarText:SetFont(font, fontsize)
		repBarText:SetShadowColor(0, 0, 0, 1)
		repBarText:SetShadowOffset(0.8, -0.8)
		repBarText:SetNonSpaceWrap(false)
		repBarText:SetJustifyH("CENTER")
		repBarText:ClearAllPoints()
		repBarText:SetAllPoints(repBar)
	end
end

function PitBull_Reputation:OnClearUnitFrame(unit, frame)
	if frame.repBar then
		frame.repBarText = delFrame(frame.repBarText)
		frame.repBar.bg = delFrame(frame.repBar.bg)
		frame.repBar = delFrame(frame.repBar)
	end
end

function PitBull_Reputation:OnUpdateStatusBarTexture(texture)
	local frame = PitBull:GetUnitFrame("player")
	if frame and frame.repBar and frame.repBar.bg then
		frame.repBar:SetStatusBarTexture(texture)
		frame.repBar.bg:SetTexture(texture)
	end
end

function PitBull_Reputation:OnUpdateFont(font, fontsize)
	local frame = PitBull:GetUnitFrame("player")
	if frame and frame.repBarText then
		frame.repBarText:SetFont(font, fontsize)
	end
end

function PitBull_Reputation:OnUnknownLayout(unit, frame, name)
	frame.repBar:SetPoint("TOPLEFT", frame, "BOTTOMLEFT")
	frame.repBar:SetPoint("TOPRIGHT", frame, "BOTTOMRIGHT")
	frame.repBar:SetHeight(20)
	frame.repBar.bg:SetAllPoints(frame.repBar)
end

local function getIgnored()
	return self.db.profile.ignore
end

local function setIgnored(value)
	self.db.profile.ignore = value
	PitBull_Reputation:UpdateReputation()
end

local function getAutoHide()
	return self.db.profile.autoHide
end

local function setAutoHide(value)
	self.db.profile.autoHide = value
	PitBull_Reputation:UpdateReputation()
end

local function getDBColor(field)
	return unpack(PitBull_Reputation.db.profile[field])
end

local function setDBColor(field,r,g,b)
	local t = PitBull_Reputation.db.profile[field]
	t[1] = r
	t[2] = g
	t[3] = b
	local frame = PitBull:GetUnitFrame("player")
	if frame then
		frame.repBarText:SetTextColor(unpack(self.db.profile.textColor))
		if not repname then
			self:UpdateReputation()
		end
	end
end

local function getDeficit()
	return PitBull_Reputation.db.profile.deficit
end
local function setDeficit(value)
	PitBull_Reputation.db.profile.deficit = value
	
	PitBull_Reputation:UpdateReputation()
end
local function getReverse()
	return PitBull_Reputation.db.profile.reverse
end
local function setReverse(value)
	PitBull_Reputation.db.profile.reverse = value
	
	PitBull_Reputation:UpdateReputation()
end


local function GetCustomColorToggle()
	return PitBull_Reputation.db.profile.customColor
end

local function SetCustomColorToggle(value)
	PitBull_Reputation.db.profile.customColor = value
	
	PitBull_Reputation:UpdateReputation()
end

local function GetCustomColor()
	return unpack(PitBull_Reputation.db.profile.customColors)
end

local function SetCustomColor(r, g, b)
	PitBull_Reputation.db.profile.customColors = {r, g, b}
	
	PitBull_Reputation:UpdateReputation()
end

local function GetCustomBGColorToggle()
	return PitBull_Reputation.db.profile.customBGColor
end

local function SetCustomBGColorToggle(value)
	PitBull_Reputation.db.profile.customBGColor = value
	
	PitBull_Reputation:UpdateReputation()
end

local function GetCustomBGColor()
	return unpack(PitBull_Reputation.db.profile.customBGColors)
end

local function SetCustomBGColor(r, g, b)
	PitBull_Reputation.db.profile.customBGColors = {r, g, b}
	
	PitBull_Reputation:UpdateReputation()
end

PitBull.options.args.global.args.colors.args.reputationBar = {
	type = "group",
	name = L["Reputation bar"],
	desc = L["Set the colors for the reputation bar."],
	args = {
		backgroundColor = {
			type = 'color',
			name = L["Background"],
			desc = L["Set background color."],
			get = getDBColor,
			set = setDBColor,
			passValue = "backgroundColor",
		},
		textColor = {
			type = 'color',
			name = L["Text"],
			desc = L["Set text color."],
			get = getDBColor,
			set = setDBColor,
			passValue = "textColor",
		},
	},
}

PitBull_Reputation:RegisterPitBullOptionsMethod(function(group)
	if group == "player" then 
		return {
			name = L["Reputation bar"],
			desc = L["Options for the units reputation bar."],
			type = "group",
			args = {
				ignore = {
					type = 'toggle',
					name = L["Disable"],
					desc = L["Disable the reputation bar."],
					get = getIgnored,
					set = setIgnored,
				},
				autoHide = {
					type = 'toggle',
					name = L["Auto hide"],
					desc = L["Hide the reputation bar when no faction is selected."],
					get = getAutoHide,
					set = setAutoHide,
					disabled = getIgnored,
				},
				deficit = {
					type = "toggle",
					name = L["Show as deficit"],
					desc = L["Show reputation deficit, draining the bar when you gain reputation."],
					get = getDeficit,
					set = setDeficit,
					disabled = getIgnored,
				},
				reverse = {
					type = "toggle",
					name = L["Reverse"],
					desc = L["Reverses the direction of the reputation bar, filling it from right to left."],
					get = getReverse,
					set = setReverse,
					disabled = getIgnored,
				},
				customColor = {
					type = "group",
					name = L["Custom color"],
					desc = L["Set a custom color for the reputation bar."],
					disabled = getIgnored,
					args = {
						Enable = {
							type = "toggle",
							name = L["Enable"],
							desc = L["Enable using a custom color for the reputation bar."],
							get = GetCustomColorToggle,
							set = SetCustomColorToggle,
						},
						setColor = {
							type = "color",
							name = L["Color"],
							desc = L["Set the custom color for the reputation bar."],
							get = GetCustomColor,
							set = SetCustomColor,
							disabled = function() return not GetCustomColorToggle() end,
						},
					},
				},
				customBGColor = {
					type = "group",
					name = L["Custom background color"],
					desc = L["Set a custom background color for the reputation bar."],
					disabled = getIgnored,
					args = {
						Enable = {
							type = "toggle",
							name = L["Enable"],
							desc = L["Enable using a custom background color for the reputation bar."],
							get = GetCustomBGColorToggle,
							set = SetCustomBGColorToggle,
						},
						setColor = {
							type = "color",
							name = L["Color"],
							desc = L["Set the custom background color for the reputation bar."],
							get = GetCustomBGColor,
							set = SetCustomBGColor,
							disabled = function() return not GetCustomBGColorToggle() end,
						},
					},
				},
			}
		}
	else
		return
	end
end)
